@@ -4,27 +4,40 @@ import sys
 from pathlib import Path
 
 import pytest
-from inline_snapshot._problems import report_problems
-from inline_snapshot.pydantic_fix import pydantic_fix
+from executing import is_pytest_compatible
 from rich import box
 from rich.console import Console
 from rich.panel import Panel
 from rich.prompt import Confirm
 from rich.syntax import Syntax
 
+from inline_snapshot._problems import report_problems
+from inline_snapshot.pydantic_fix import pydantic_fix
+
 from . import _config
 from . import _external
 from . import _find_external
-from . import _inline_snapshot
 from ._change import apply_all
 from ._code_repr import used_hasrepr
 from ._find_external import ensure_import
+from ._flags import Flags
+from ._global_state import snapshot_env
+from ._global_state import state
 from ._inline_snapshot import used_externals
 from ._rewrite_code import ChangeRecorder
+from ._snapshot.generic_value import GenericValue
 
 pytest.register_assert_rewrite("inline_snapshot.extra")
 pytest.register_assert_rewrite("inline_snapshot.testing._example")
 
+if sys.version_info >= (3, 13):
+    # fixes #186
+    try:
+        import readline  # noqa
+    except ModuleNotFoundError:  # pragma: no cover
+        # should fix #189
+        pass
+
 
 def pytest_addoption(parser, pluginmanager):
     group = parser.getgroup("inline-snapshot")
@@ -58,7 +71,7 @@ def pytest_addoption(parser, pluginmanager):
             )
 
 
-categories = {"create", "update", "trim", "fix"}
+categories = Flags.all().to_set()
 flags = set()
 
 
@@ -105,27 +118,25 @@ def pytest_configure(config):
         )
 
     if xdist_running(config) or not is_implementation_supported():
-        _inline_snapshot._active = False
+        state().active = False
 
     elif flags & {"review"}:
-        _inline_snapshot._active = True
+        state().active = True
 
-        _inline_snapshot._update_flags = _inline_snapshot.Flags(
-            {"fix", "create", "update", "trim"}
-        )
+        state().update_flags = Flags.all()
     else:
 
-        _inline_snapshot._active = "disable" not in flags
+        state().active = "disable" not in flags
 
-        _inline_snapshot._update_flags = _inline_snapshot.Flags(flags & categories)
+        state().update_flags = Flags(flags & categories)
 
     external_storage = (
         _config.config.storage_dir or config.rootpath / ".inline-snapshot"
     ) / "external"
 
-    _external.storage = _external.DiscStorage(external_storage)
+    state().storage = _external.DiscStorage(external_storage)
 
-    if flags - {"short-report", "disable"}:
+    if flags - {"short-report", "disable"} and not is_pytest_compatible():
 
         # hack to disable the assertion rewriting
         # I found no other way because the hook gets installed early
@@ -135,29 +146,36 @@ def pytest_configure(config):
 
     pydantic_fix()
 
-    _external.storage.prune_new_files()
+    state().storage.prune_new_files()
 
 
 @pytest.fixture(autouse=True)
-def snapshot_check():
-    _inline_snapshot._missing_values = 0
-    _inline_snapshot._incorrect_values = 0
-    yield
+def snapshot_check(request):
+    state().missing_values = 0
+    state().incorrect_values = 0
 
-    missing_values = _inline_snapshot._missing_values
-    incorrect_values = _inline_snapshot._incorrect_values
+    if "xfail" in request.keywords:
+        with snapshot_env() as local_state:
+            local_state.active = False
+            yield
+        return
+    else:
+        yield
 
-    if missing_values != 0 and not _inline_snapshot._update_flags.create:
+    missing_values = state().missing_values
+    incorrect_values = state().incorrect_values
+
+    if missing_values != 0 and not state().update_flags.create:
         pytest.fail(
             (
-                f"your snapshot is missing one value."
+                "your snapshot is missing one value."
                 if missing_values == 1
                 else f"your snapshot is missing {missing_values} values."
             ),
             pytrace=False,
         )
 
-    if incorrect_values != 0 and not _inline_snapshot._update_flags.fix:
+    if incorrect_values != 0 and not state().update_flags.fix:
         pytest.fail(
             "some snapshots in this test have incorrect values.",
             pytrace=False,
@@ -165,13 +183,14 @@ def snapshot_check():
 
 
 def pytest_assertrepr_compare(config, op, left, right):
+
     results = []
-    if isinstance(left, _inline_snapshot.GenericValue):
+    if isinstance(left, GenericValue):
         results = config.hook.pytest_assertrepr_compare(
             config=config, op=op, left=left._visible_value(), right=right
         )
 
-    if isinstance(right, _inline_snapshot.GenericValue):
+    if isinstance(right, GenericValue):
         results = config.hook.pytest_assertrepr_compare(
             config=config, op=op, left=left, right=right._visible_value()
         )
@@ -218,7 +237,7 @@ def pytest_terminal_summary(terminalreporter, exitstatus, config):
             )
         return
 
-    if not _inline_snapshot._active:
+    if not state().active:
         return
 
     terminalreporter.section("inline snapshot")
@@ -243,7 +262,7 @@ def pytest_terminal_summary(terminalreporter, exitstatus, config):
             console.print()
             return result
         else:
-            console.print(f"These changes are not applied.")
+            console.print("These changes are not applied.")
             console.print(
                 f"Use [b]--inline-snapshot={flag}[/] to apply them, or use the interactive mode with [b]--inline-snapshot=review[/]",
                 highlight=False,
@@ -252,21 +271,11 @@ def pytest_terminal_summary(terminalreporter, exitstatus, config):
             return False
 
     # auto mode
-    changes = {
-        "update": [],
-        "fix": [],
-        "trim": [],
-        "create": [],
-    }
+    changes = {f: [] for f in Flags.all()}
 
-    snapshot_changes = {
-        "update": 0,
-        "fix": 0,
-        "trim": 0,
-        "create": 0,
-    }
+    snapshot_changes = {f: 0 for f in Flags.all()}
 
-    for snapshot in _inline_snapshot.snapshots.values():
+    for snapshot in state().snapshots.values():
         all_categories = set()
         for change in snapshot._changes():
             changes[change.flag].append(change)
@@ -285,7 +294,7 @@ def pytest_terminal_summary(terminalreporter, exitstatus, config):
             def report(flag, message, message_n):
                 num = snapshot_changes[flag]
 
-                if num and not getattr(_inline_snapshot._update_flags, flag):
+                if num and not getattr(state().update_flags, flag):
                     console.print(
                         message if num == 1 else message.format(num=num),
                         highlight=False,
@@ -323,12 +332,13 @@ def pytest_terminal_summary(terminalreporter, exitstatus, config):
 
             return
 
-        assert not any(
-            type(e).__name__ == "AssertionRewritingHook" for e in sys.meta_path
-        )
+        if not is_pytest_compatible():
+            assert not any(
+                type(e).__name__ == "AssertionRewritingHook" for e in sys.meta_path
+            )
 
         used_changes = []
-        for flag in ("create", "fix", "trim", "update"):
+        for flag in Flags.all():
             if not changes[flag]:
                 continue
 
@@ -337,64 +347,66 @@ def pytest_terminal_summary(terminalreporter, exitstatus, config):
 
             console.rule(f"[yellow bold]{flag.capitalize()} snapshots")
 
-            with ChangeRecorder().activate() as cr:
-                apply_all(used_changes)
-                cr.virtual_write()
-                apply_all(changes[flag])
+            cr = ChangeRecorder()
+            apply_all(used_changes, cr)
+            cr.virtual_write()
+            apply_all(changes[flag], cr)
 
-                for file in cr.files():
-                    diff = file.diff()
-                    if diff:
-                        name = file.filename.relative_to(Path.cwd())
-                        console.print(
-                            Panel(
-                                Syntax(diff, "diff", theme="ansi_light"),
-                                title=str(name),
-                                box=(
-                                    box.ASCII
-                                    if os.environ.get("TERM", "") == "unknown"
-                                    else box.ROUNDED
-                                ),
-                            )
+            for file in cr.files():
+                diff = file.diff()
+                if diff:
+                    name = file.filename.relative_to(Path.cwd())
+                    console.print(
+                        Panel(
+                            Syntax(diff, "diff", theme="ansi_light"),
+                            title=str(name),
+                            box=(
+                                box.ASCII
+                                if os.environ.get("TERM", "") == "unknown"
+                                else box.ROUNDED
+                            ),
                         )
+                    )
 
-                if apply_changes(flag):
-                    used_changes += changes[flag]
+            if apply_changes(flag):
+                used_changes += changes[flag]
 
         report_problems(console)
 
         if used_changes:
-            with ChangeRecorder().activate() as cr:
-                apply_all(used_changes)
+            cr = ChangeRecorder()
+            apply_all(used_changes, cr)
 
-                for test_file in cr.files():
-                    tree = ast.parse(test_file.new_code())
-                    used = used_externals(tree)
+            for test_file in cr.files():
+                tree = ast.parse(test_file.new_code())
+                used = used_externals(tree)
 
-                    required_imports = []
+                required_imports = []
 
-                    if used:
-                        required_imports.append("external")
+                if used:
+                    required_imports.append("external")
 
-                    if used_hasrepr(tree):
-                        required_imports.append("HasRepr")
+                if used_hasrepr(tree):
+                    required_imports.append("HasRepr")
 
-                    if required_imports:
-                        ensure_import(
-                            test_file.filename, {"inline_snapshot": required_imports}
-                        )
+                if required_imports:
+                    ensure_import(
+                        test_file.filename,
+                        {"inline_snapshot": required_imports},
+                        cr,
+                    )
 
-                    for external_name in used:
-                        _external.storage.persist(external_name)
+                for external_name in used:
+                    state().storage.persist(external_name)
 
-                cr.fix_all()
+            cr.fix_all()
 
         unused_externals = _find_external.unused_externals()
 
-        if unused_externals and _inline_snapshot._update_flags.trim:
+        if unused_externals and state().update_flags.trim:
             for name in unused_externals:
-                assert _external.storage
-                _external.storage.remove(name)
+                assert state().storage
+                state().storage.remove(name)
             terminalreporter.write(
                 f"removed {len(unused_externals)} unused externals\n"
             )
